#ifndef PARTICLE_HPP
#define PARTICLE_HPP

#include <iostream>
using std::ostream;
using std::istream;

#include <cmath>
using std::sqrt;

#include <string>
using std::string;

#include "vec2.hpp"

class particle
{
public:
	// member data
	string name;
	double mass;
	vec2 r; // position
	vec2 v; // velocity
	vec2 a; // acceleration
	double t; // current time;

	// constructor
	particle() : name(string("_")), mass(0), r(), v(), a(), t(0) {}
	particle(double m_, vec2 r_, vec2 v_) : name(string("_")), mass(m_), r(r_), v(v_), a(), t(0) {}
	particle(string n_, double m_, vec2 r_, vec2 v_) : name(n_), mass(m_), r(r_), v(v_), a(), t(0) {}
	particle(const char n_[], double m_, vec2 r_, vec2 v_) : name(n_), mass(m_), r(r_), v(v_), a(), t(0) {}
	
	// methods (or member functions)
	void evolve(void (&method)(particle&,double), double dt)
	{
		// *this refers to the active particle object
		method(*this, dt);
	}

#ifdef USE_OPENGL
	double draw(void)
	{
		const double radius = 0.03*pow(mass,0.25);
		if (radius < 0.005)
		{
			glPointSize(2.0); 
			glBegin(GL_POINTS); 
			glColor3f(1.0, 1.0, 1.0); 
			glVertex2f(r.x,r.y);
			glEnd();
			glFlush();
			return 0.005;
		}
		else
		{
			glBegin(GL_LINE_LOOP); 
			glColor3f(1.0, 0.5, 0.25); 
			for(int j = 0; j < 360; ++j)
				glVertex2f(r.x+radius*cos(j*M_PI/180.0),r.y+radius*sin(j*M_PI/180.0));
			glEnd();
			glFlush();
			return radius;
		}
	}
#endif
	
	// a friend function does not belong to the class but is affiliated with it
	friend ostream& operator<<(ostream& os, const particle &p)
	{
		os.precision(8);
		os << setw(20) << p.name
		   << setw(20) << p.t
		   << setw(20) << p.r
		   << setw(20) << p.v;
		return os;
	}
	friend istream& operator>>(istream& is, particle &p)
	{
		is >> p.name >> p.mass >> p.t >> p.r >> p.v;
		return is;
	}	
#ifdef USE_OPENGL	
	friend void connect(const particle &p1, const particle &p2)
	{
		glPointSize(3.0);
		glBegin(GL_LINES); 
		   glColor3f(1.0, 1.0, 1.0); 
		   glVertex2f(p1.r.x,p1.r.y);
		   glVertex2f(p2.r.x,p2.r.y);
		glEnd();
	}
	friend void anticonnect(const particle &p1, const particle &p2)
	{
		const double dx = p2.r.x - p1.r.x;
		const double dy = p2.r.y - p1.r.y;
		glPointSize(3.0);
		glBegin(GL_LINES); 
		   glColor3f(1.0, 1.0, 1.0);
		   glVertex2f(p1.r.x,p1.r.y);
		   glVertex2f(p1.r.x-dx,p1.r.y-dy);
		glEnd();
		glBegin(GL_LINES); 
		   glColor3f(1.0, 1.0, 1.0);
		   glVertex2f(p2.r.x,p2.r.y);
		   glVertex2f(p2.r.x+dx,p2.r.y+dy);
		glEnd();
	}
#endif	
};

#endif // PARTICLE_HPP
